@@ -610,4 +610,11 @@ int _gcry_mpi_get_flag (gcry_mpi_t a, enum gcry_mpi_flag flag);
 #define mpi_get_flag(a,f)      _gcry_mpi_get_flag ((a), (f))
 
 
+enum gcry_mldsa_algos
+  {                             /* See FIPS 204, Table 1 */
+    GCRY_MLDSA44,               /* Category 2 */
+    GCRY_MLDSA65,               /* Category 3 */
+    GCRY_MLDSA87                /* Category 5 */
+  };
+
 #endif /*GCRY_GCRYPT_INT_H*/
